<?PHP
/**
 * 唯品会JIT采购单任务(每小时执行一次)
 *
 * @author wangbiao@shopex.cn
 * @version 0.1
 */

class ome_autotask_timer_vopurchase
{
    /* 当前的执行时间 */
    public static $now;
    
    /* 执行的间隔时间 */
    const intervalTime = 3600;
    
    function __construct()
    {
        self::$now = time();
    }
    
    /**
     * 根据供应商获取po列表
     */
    public function process($params, &$error_msg='')
    {
        @set_time_limit(0);
        @ini_set('memory_limit','128M');
        ignore_user_abort(1);
        
        base_kvstore::instance('console/vop/purchase')->fetch('apply-lastexectime',$lastExecTime);
        if($lastExecTime && ($lastExecTime+self::intervalTime)>self::$now) {
            return false;
        }
        
        base_kvstore::instance('console/vop/purchase')->store('apply-lastexectime', self::$now);
        
        $setObj    = app::get('purchase')->model('setting');
        
        //获取配置的店铺列表
        $sql       = "SELECT * FROM sdb_purchase_setting_shop WHERE 1 GROUP BY shop_id";
        $tempData  = $setObj->db->select($sql);
        
        $shop_ids    = array();
        if($tempData)
        {
            foreach ($tempData as $key => $val)
            {
                $shop_ids[]    = $val['shop_id'];
            }
        }
        if(empty($shop_ids))
        {
            return false;
        }
        
        //获取唯品会店铺
        $shopObj    = app::get('ome')->model('shop');
        $shopList   = $shopObj->getList('shop_id, shop_bn, name', array('node_type'=>'vop', 'node_id|noequal'=>'', 'shop_id'=>$shop_ids));
        if(empty($shopList))
        {
            return false;
        }
        
        foreach ($shopList as $key => $shop_info)
        {
            //[拉取PO单]档期开始销售时间(一个月内)
            $retry   = array();
            $page    = 1;
            do {
                //超时计数器
                if(empty($retry[$page]))
                {
                    $retry[$page]    = 1;
                }
                
                //组织数据
                $month_time    = strtotime('-1 month');
                $param         = array('st_sell_st_time' => date('Y-m-d H:i:s', $month_time),
                                        'et_sell_st_time' => date('Y-m-d H:i:s', self::$now),
                                        'page' => $page,
                                        'limit' => 50,
                                );
                $rsp_data = kernel::single('erpapi_router_request')->set('shop', $shop_info['shop_id'])->purchase_getPoList($param);
                
                //拉取拣货单列表
                if($rsp_data['rsp'] == 'succ')
                {
                    //没有数据跳出
                    if($rsp_data['is_empty'])
                    {
                        break;
                    }
                    
                    //拉取拣货单列表
                    if($rsp_data['po_nos'])
                    {
                        $this->getPickList($rsp_data['po_nos'], $shop_info['shop_id']);
                    }
                    
                    $page++;
                }
                elseif(($rsp_data['rsp']=='fail') && ($rsp_data['err_msg']=='请求超时'))
                {
                    //重试三次后跳出
                    if($retry[$page] > 3)
                    {
                        break;
                    }
                    $retry[$page]++;
                }
                else 
                {
                    break;//接口报错跳出
                }
            } while (true);
        }
        
        return true;
    }
    
    /**
     * 拉取拣货单列表
     * 
     * @param Array $po_nos  采购单bn
     * @param string $shop_id
     * 
     * @return Array $pick_list 拣货单信息列表
     */
    public function getPickList($bns, $shop_id)
    {
        $pick_list = array();
        $po_nos    = implode(',', $bns);
        $param     = array('po_no'=>$po_nos);
        
        $rsp_data  = kernel::single('erpapi_router_request')->set('shop', $shop_id)->purchase_getPickList($param);
        
        //拉取拣货单明细
        if($rsp_data['rsp'] == 'succ' && $rsp_data['po_list'])
        {
            foreach ($rsp_data['po_list'] as $pi_key => $pi_val)
            {
                $param       = array('po_no'=>$pi_val['po_no'], 'pick_no'=>$pi_val['pick_no']);
                $rsp_detail  = kernel::single('erpapi_router_request')->set('shop', $shop_id)->purchase_getPickDetail($param);
                
                //组织拣货单信息
                if($rsp_detail['rsp'] == 'succ' && $rsp_detail['pick_product_list'])
                {
                    $pick_list[$pi_key]    = $pi_val;
                    $pick_list[$pi_key]['product_list']    = $rsp_detail['pick_product_list'];
                }
            }
        }
        
        //保存拣货单
        if($pick_list)
        {
            $pickLib   = kernel::single('purchase_purchase_pick');
            $result    = $pickLib->save_pick($pick_list);
        }
        
        return true;
    }
}